﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace WinGrid_OneBigButtonColumnCS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            this.ultraGrid1.CreationFilter = new MyCreationFilter();

            // We need a DrawFilter to hide the focus rect or els it looks weird going through the button. 
            this.ultraGrid1.DrawFilter = new NoFocusRect();

            this.uiElementViewerDialog1.Show(this, new Point(this.Bounds.Right, this.Bounds.Top));
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            UltraGridLayout layout = e.Layout;
            UltraGridBand band = layout.Bands[0];
            UltraGridOverride ov = layout.Override;

            ov.RowSelectors = DefaultableBoolean.False;
            
            if (false == band.Columns.Exists("ButtonColumn"))
            {
                UltraGridColumn buttonColumn = band.Columns.Add("ButtonColumn");
                buttonColumn.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.Image;
                buttonColumn.Header.VisiblePosition = 0;
                buttonColumn.MergedCellStyle = MergedCellStyle.Always;                
            }
        }
       
        public class MyCreationFilter : IUIElementCreationFilter
        {
            bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
            {
                // Watch or the merged cell of the unbound button column. 
                MergedCellUIElement mergedCellUIElement = parent as MergedCellUIElement;
                if (null != mergedCellUIElement &&
                    mergedCellUIElement.Column.Key == "ButtonColumn")
                {
                    // Get the ButtonUIElement, if there is one, or if not, create  a new one. 
                    ButtonUIElement buttonUIElement = mergedCellUIElement.GetDescendant(typeof(ButtonUIElement)) as ButtonUIElement;
                    if (null == buttonUIElement)
                    {
                        buttonUIElement = new ButtonUIElement(parent);
                        parent.ChildElements.Add(buttonUIElement);
                    }

                    // Set the rect of the button element to the merhed cell rect. 
                    buttonUIElement.Rect = parent.Rect;
                    return true;
                }

                // Do nothing
                return false;
            }

            void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
            {
                 BandHeadersUIElement bandHeadersUIElement = parent as BandHeadersUIElement;
                if (null != bandHeadersUIElement)
                {
                    // Get the unbound button column. 
                    UltraGridBase grid = (UltraGridBase)parent.Control;
                    UltraGridBand band = grid.DisplayLayout.Bands[0];
                    UltraGridColumn buttonColumn = band.Columns.Exists("ButtonColumn")
                        ? band.Columns["ButtonColumn"]
                        : null;

                    // If the button column does not exist, bail out and do nothing. 
                    if (null == buttonColumn)
                        return;

                    // Remove the column header for the button column. 
                    HeaderUIElement buttonColumnHeaderElement = bandHeadersUIElement.GetDescendant(typeof(HeaderUIElement), buttonColumn) as HeaderUIElement;
                    if (null != buttonColumnHeaderElement)
                    {
                        bandHeadersUIElement.ChildElements.Remove(buttonColumnHeaderElement);
                    }
                }
            }            
        }
    }

    public class NoFocusRect : Infragistics.Win.IUIElementDrawFilter
    {
        #region Implementation of IUIElementDrawFilter
        public bool DrawElement(Infragistics.Win.DrawPhase drawPhase, ref Infragistics.Win.UIElementDrawParams drawParams)
        {
            return true;
        }
        public Infragistics.Win.DrawPhase GetPhasesToFilter(ref Infragistics.Win.UIElementDrawParams drawParams)
        {
            return Infragistics.Win.DrawPhase.BeforeDrawFocus;
        }
        #endregion
    }
}
